/*
    Valtra Tractors
    Copyright 2004 Martin Storsj

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Martin Storsj
    martin@martin.st
*/

#include <stdlib.h>
#include <time.h>
#include <SDL.h>
#include <SDL_image.h>
#include <SDL_ttf.h>
#include <math.h>
#include <algorithm>
#include "board.h"
#include "spline.h"
#include "fsys.h"



//#define SPLINETEST

//#define MOVESYNCEND





#ifndef M_PI
#define M_PI 3.141592654
#endif

using std::min;
using std::max;


#define HEXTILEWIDTH 77
#define HEXTILEHEIGHT 66

const int origTileNumbers[] = {
	9, 10, 14, 5, 5, 5, 5, 5, 5, 5
};

const FPoint tOffset[4] = {
	FPoint(0,0),
	FPoint(0,0),
	FPoint(0,0),
	FPoint(0,0)
};

const IPoint textPos[4] = {
	IPoint(45,59),
	IPoint(600,59),
	IPoint(600,518),
	IPoint(45,518)
};

const FPoint tileOffset[11] = {
	FPoint(0,0),
	FPoint(-10,2),
	FPoint(-11,-15),
	FPoint(0,0),
	FPoint(-10,0),
	FPoint(0,0),
	FPoint(0,0),
	FPoint(-3,0),
	FPoint(0,0),
	FPoint(0,0),
	FPoint(0,0) // endTile
};

const IPoint pointerOff[] = {
	IPoint(1,1),
	IPoint(1,1),
	IPoint(2,1)
};



const TilePos connections[][6] = {
	{ DOWN, UP, THIS, THIS, THIS, THIS },
	{ DOWN, UPLEFT, THIS, THIS, THIS, THIS },
	{ UP, UPLEFT, THIS, THIS, THIS, THIS },
	{ UPLEFT, DOWNRIGHT, DOWNLEFT, UPRIGHT, THIS, THIS },
	{ DOWN, UPLEFT, UP, DOWNLEFT, THIS, THIS },
	{ DOWN, UP, UPLEFT, UPRIGHT, DOWNLEFT, DOWNRIGHT },
	{ DOWN, UP, UPLEFT, UPRIGHT, THIS, THIS },
	{ DOWN, UP, UPLEFT, DOWNLEFT, THIS, THIS },
	{ DOWN, UP, UPLEFT, DOWNRIGHT, DOWNLEFT, UPRIGHT },
	{ DOWN, UPLEFT, DOWNLEFT, DOWNRIGHT, UP, UPRIGHT }
};


#define STARTTILE 42
#define ENDTILE 43


TilePos operator++(TilePos &pos, int) {
/*	switch (pos) {
	case THIS: return UPLEFT;
	case UPLEFT: return UP;
	case UP: return UPRIGHT;
	case UPRIGHT: return DOWNLEFT;
	case DOWNLEFT: return DOWN;
	case DOWN: return DOWNRIGHT;
	case DOWNRIGHT: return DOWNRIGHT+1;
	}
	return THIS;
*/
	TilePos old = pos;
	pos = TilePos(pos+1);
	return old;
}

TilePos BoardGame::rotateDir(TilePos in, int n) {
	if (in==THIS) return THIS;
	if (n>=6) n -= 6*(n/6);
	if (n<0) n += 6*(-(n-5)/6);
	while (n>0) {
		if (in!=DOWNLEFT) in++;
		else in = UPLEFT;
		n--;
	}
	return in;
}

TilePos BoardGame::oppositeDir(TilePos in) {
	switch (in) {
	case UP: return DOWN;
	case DOWN: return UP;
	case UPLEFT: return DOWNRIGHT;
	case UPRIGHT: return DOWNLEFT;
	case DOWNLEFT: return UPRIGHT;
	case DOWNRIGHT: return UPLEFT;
	default: return THIS;
	}
}

void BoardGame::rotateTile(int nr, int steps, TilePos *array) {
	for (int i = 0; i < 6; i++)
		array[i] = rotateDir(connections[nr][i],steps);
}




int BoardGame::height(int x) {
	if (x%2!=0) return 8;
	return 7;
}

bool BoardGame::coordsOk(int x, int y) {
	if (x<0) return false;
	if (y<0) return false;
	if (y>=height(x)) return false;
	if (x>=9) return false;
/*	if (x==0 || x==8) {
		if (y==0) return false;
		if (y==6) return false;
	}
*/
	if (grid[x][y].first==STARTTILE) return false; // unused starting point
	if (grid[x][y].first==ENDTILE) return false;
	return true;
}

void BoardGame::nextPos(int &gridx, int &gridy, TilePos dir) {
	switch (dir) {
	case UP: gridy--; break;
	case DOWN: gridy++; break;
	case UPLEFT: if (gridx--%2!=0) gridy--; break;
	case UPRIGHT: if (gridx++%2!=0) gridy--; break;
	case DOWNLEFT: if (gridx--%2==0) gridy++; break;
	case DOWNRIGHT: if (gridx++%2==0) gridy++; break;
	default: break;
	}
}

int BoardGame::getRotation(TilePos dir) {
	switch (dir) {
	case UP: return 0;
	case UPRIGHT: return 1;
	case DOWNRIGHT: return 2;
	case DOWN: return 3;
	case DOWNLEFT: return 4;
	case UPLEFT: return 5;
	default: return 0;
	}
}

/*
#define TILEDISTX 57.5
#define TILEDISTY 66.5
#define GRIDSTARTX 60
#define GRIDSTARTY 66
#define TILEOFFSETY 33
*/

#define TILEDISTX 58.0
#define TILEDISTY 66.5
#define GRIDSTARTX 52
#define GRIDSTARTY 58
#define TILEOFFSETY 33

void BoardGame::getCoords(int gridx, int gridy, Sint16 &x, Sint16 &y) {
	x = (int)(gridx*TILEDISTX) + GRIDSTARTX;
	y = (int)(gridy*TILEDISTY) + GRIDSTARTY - (gridx%2!=0)*TILEOFFSETY;
}

void BoardGame::getDrawRect(int gridx, int gridy, SDL_Surface *img, SDL_Rect &rect) {
	getCoords(gridx,gridy,rect.x,rect.y);
//	rect.x -= (img->w-tile[0]->w)/2;
//	rect.y -= (img->h-tile[0]->h)/2;
	rect.x -= (img->w-HEXTILEWIDTH)/2;
	rect.y -= (img->h-HEXTILEHEIGHT)/2;
	rect.w = img->w;
	rect.h = img->h;
}

void BoardGame::getDrawRect(int gridx, int gridy, const SDL_Rect &img, SDL_Rect &rect) {
	getCoords(gridx,gridy,rect.x,rect.y);
//	rect.x -= (img.w-tile[0]->w)/2;
//	rect.y -= (img.h-tile[0]->h)/2;
	rect.x -= (img.w-HEXTILEWIDTH)/2;
	rect.y -= (img.h-HEXTILEHEIGHT)/2;
	rect.w = img.w;
	rect.h = img.h;
}


void BoardGame::getGridCoords(int x, int y, int &gridx, int &gridy) {
	gridx = 0;
	gridy = 0;
	Sint16 curX, curY;
	getCoords(gridx,gridy,curX,curY);
	if (x>curX) {
		while (x>curX) {
			gridx++;
			getCoords(gridx,gridy,curX,curY);
		}
		gridx--;
		getCoords(gridx,gridy,curX,curY);
	} else {
		while (x<curX) {
			gridx--;
			getCoords(gridx,gridy,curX,curY);
		}
	}
	if (y>curY) {
		while (y>curY) {
			gridy++;
			getCoords(gridx,gridy,curX,curY);
		}
		gridy--;
		getCoords(gridx,gridy,curX,curY);
	} else {
		while (y<curY) {
			gridy--;
			getCoords(gridx,gridy,curX,curY);
		}
	}

	TilePos pos = getPos(x,y,curX,curY);
	switch (pos) {
	case UPLEFT:
		if (gridx%2) gridy--;
		gridx--;
		break;
	case DOWNLEFT:
		if (gridx%2==0) gridy++;
		gridx--;
		break;
	case UPRIGHT:
		if (gridx%2) gridy--;
		gridx++;
		break;
	case DOWNRIGHT:
		if (gridx%2==0) gridy++;
		gridx++;
		break;
	default:
		break;
	}
}

TilePos BoardGame::getPos(int x, int y, int tilex, int tiley) {
	// hardcoded coords!
	x -= tilex;
	y -= tiley;

	if (y<0) return UP;
	if (y>66) return DOWN;

	if (x<35) {
		if (y<33) {
			int borderX = 18*(32-y)/32;
			if (x<borderX)
				return UPLEFT;
		} else {
			int borderX = 18*(y-33)/32;
			if (x<borderX)
				return DOWNLEFT;
		}
	} else {
		if (y<33) {
			int borderX = 76 - 19*(32-y)/32;
			if (x>borderX)
				return UPRIGHT;
		} else {
			int borderX = 76 - 19*(y-33)/32;
			if (x>borderX)
				return DOWNRIGHT;
		}
	}
	return THIS;
}

bool BoardGame::isInsideTile(int x, int y, int tilex, int tiley) {
	TilePos pos = getPos(x,y,tilex,tiley);
	return pos==THIS;
}

Game2 *constructBoardGame(SDL_Surface *screen, Uint32 time) {
	return new BoardGame(screen, time);
}

void BoardGame::startGame(int players) {
	SDL_BlitSurface(board,NULL,bkg,NULL);

	for (int i = 0; i < 4; i++) {
		PlayerStatus &ps = player[i];
		if (ps.rotImg!=NULL) {
			SDL_FreeSurface(ps.rotImg);
			ps.rotImg = NULL;
		}
		if (ps.curSpline!=NULL) {
			delete ps.curSpline;
			ps.curSpline = NULL;
		}
		ps = PlayerStatus();
	}

	if (isCoded(players)) {
		for (int i = 0; i < 4; i++) {
			if (hasPlayer(players,i)) {
				player[i].active = true;
			}
		}
		players = getPlayers(players);
	} else {
		if (players<0) players = 0;
		if (players>4) players = 4;
		switch (players) {
		case 4:
			player[3].active = true;
		case 3:
			player[1].active = true;
		case 2:
			player[2].active = true;
		case 1:
			player[0].active = true;
		case 0:
			break;
		}
	}

	for (int i = 0; i < 4; i++) {
		if (!player[i].active) continue;
		switch (i) {
		case 0:
//			player[0] = PlayerStatus(0,0,DOWNRIGHT,0x00DAA37F);
			player[0] = PlayerStatus(0,0,DOWNRIGHT,0x00CF8555);
			break;
		case 1:
			player[1] = PlayerStatus(8,0,DOWNLEFT,0x00EEEFF0);
			break;
		case 2:
			player[2] = PlayerStatus(8,6,UPLEFT,0x00CC2238);
			break;
		case 3:
			player[3] = PlayerStatus(0,6,UPRIGHT,0x007B88A9);
			break;
		default:
			break;
		}
	}


	animStartTime = lastTime;
	state = WAITING;
	if (players==0) state = EMPTY;
	selectedTile = -1;
	tileRotation = 0;
	curTile = NULL;
	for (int x = 0; x < GRIDWIDTH; x++)
		for (int y = 0; y < GRIDHEIGHT; y++)
			grid[x][y].first = -1;
	grid[0][0] = pair<int,int>(player[0].active?0:STARTTILE,2);
	grid[8][0] = pair<int,int>(player[1].active?0:STARTTILE,1);
	grid[8][6] = pair<int,int>(player[2].active?0:STARTTILE,2);
	grid[0][6] = pair<int,int>(player[3].active?0:STARTTILE,1);
	tileIsOk = false;
	problemHighlight = NULL;
	while (true) {
		curPlayer = rand()%4;
		if (players==0) break;
		if (player[curPlayer].active) break;
	}


	for (int i = 0; i < 10; i++) {
		SDL_Rect pos;
		pos.x = tilePos[i].x;
		pos.y = tilePos[i].y;
		SDL_BlitSurface(tileBkg,NULL,bkg,&pos);
		addTileOffset(pos,i,0,false);
		SDL_BlitSurface(tile[i],NULL,bkg,&pos);
		numTiles[i] = origTileNumbers[i];
	}

	for (int i = 0; i < 3; i++)
		SDL_BlitSurface(buttons[i].surf[0],NULL,bkg,&buttons[i].r);

	buttondown = buttonover = -1;
	buttondownTile = -1;

	redrawLabels = true;
	redrawLabels2 = true;

	quit = false;

	isPaused = false;


	last = *this;

	se.redrawAll();
}

BoardGame::BoardGame(SDL_Surface *screen, Uint32 curtime) : Game2(screen, curtime) {
	bkg = SDL_CreateRGBSurface(SDL_HWSURFACE,screen->w,screen->h,32,screen->format->Rmask,screen->format->Gmask,screen->format->Bmask,0);
	SDL_FillRect(bkg,NULL,SDL_MapRGB(bkg->format,0x1C,0xB8,0x14));

	Effects::precalcHighlight(&highlightSize);

	lastTime = curtime;



	for (int i = 0; i < 3; i++) {
		for (int j = 0; j < 3; j++) {
			char buffer[50];
			sprintf(buffer,"button%d-%d.png",i+1,j+1);
			buttons[i].surf[j] = IMG_Load(buffer);
		}
		buttons[i].setClickmap(buttons[i].surf[0]);
		buttons[i].r.w = buttons[i].surf[0]->w;
		buttons[i].r.h = buttons[i].surf[0]->h;
	}
	buttons[0].r.x = 637;
	buttons[0].r.y = 494;
	buttons[1].r.x = 690;
	buttons[1].r.y = 494;
	buttons[2].r.x = 737;
	buttons[2].r.y = 494;



	board = IMG_Load("board.png");
	SDL_Rect pos;
	pos.x = 0;
	pos.y = 0;
	SDL_BlitSurface(board,NULL,bkg,&pos);
	SDL_FreeSurface(board);
	board = SDL_ConvertSurface(bkg,bkg->format,SDL_HWSURFACE);

	SDL_PixelFormat fmt = ARGB_fmt;
	for (int i = 0; i < 10; i++) {
		char buffer[25];
		sprintf(buffer,"tile%d.png",i);
		SDL_Surface *tmp = IMG_Load(buffer);
		tile[i] = SDL_ConvertSurface(tmp,&fmt,SDL_SWSURFACE);
		SDL_FreeSurface(tmp);
		tilePos[i].x = 637+(i/5)*77;
		tilePos[i].y = 30+(i%5)*85+(i/5)*42;
	}
	SDL_Surface *tmpTile = IMG_Load("endtile.png");
	endTile = SDL_ConvertSurface(tmpTile,&fmt,SDL_SWSURFACE);
	SDL_FreeSurface(tmpTile);
	tileBkg = IMG_Load("tilebkg.png");

//	initSplines(tile[0]->w,tile[0]->h);
	initSplines(HEXTILEWIDTH,HEXTILEHEIGHT);

	for (int i = 0; i < 4; i++) {
		char buffer[25];
		sprintf(buffer,"tractor%d.png",i+1);
		SDL_Surface *tmp = IMG_Load(buffer);
		tractor[i] = SDL_ConvertSurface(tmp,&fmt,SDL_SWSURFACE);
		SDL_FreeSurface(tmp);

		PlayerStatus &ps = player[i];
		ps.curSpline = NULL;
	}

	pointer[0] = IMG_Load("pointer1.png");
	pointer[1] = IMG_Load("pointer2.png");
	pointer[2] = IMG_Load("pointer3.png");
	SDL_GetMouseState(&curMousePos.x,&curMousePos.y);
	curPointer = NULL;
	setPointer(0);

	TTF_Init();
	font = TTF_OpenFont("Vera.ttf",14);


	startGame(2);

}


BoardGame::~BoardGame() {
	SDL_FreeSurface(board);
	for (int i = 0; i < 10; i++)
		SDL_FreeSurface(tile[i]);
	SDL_FreeSurface(tileBkg);
	SDL_FreeSurface(endTile);
	for (int i = 0; i < 4; i++) {
		SDL_FreeSurface(tractor[i]);
		delete player[i].curSpline;
	}
	SDL_FreeSurface(bkg);
	for (int i = 0; i < 3; i++)
		SDL_FreeSurface(pointer[i]);
	TTF_CloseFont(font);
	TTF_Quit();
}

int BoardGame::findButton(int x, int y) {
	for (int i = 0; i < 3; i++) {
		if (buttons[i].isClickable(x,y))
			return i;
	}
	return -1;
}

int BoardGame::findTile(int x, int y) {
	for (int i = 0; i < 10; i++) {
		if (isInsideTile(x,y,tilePos[i].x,tilePos[i].y)) return i;
	}
	return -1;
}


bool BoardGame::advance(int &gridx, int &gridy, TilePos &dir) {
	int newx = gridx, newy = gridy;
	nextPos(newx,newy,dir);
	if (grid[newx][newy].first==-1) return false;
	TilePos directions[6];
	rotateTile(grid[newx][newy].first,grid[newx][newy].second,directions);
	TilePos opp = oppositeDir(dir);
	int i = 0;
	for (; i < 6; i++) {
		if (directions[i]==THIS) return false;
		if (directions[i]==opp) {
			gridx = newx;
			gridy = newy;
			dir = directions[i%2?i-1:i+1];
			return true;
		}
	}
	return false;
}


bool BoardGame::testAdvance(int x, int y, TilePos dir) {
	while (true) {
		nextPos(x,y,dir);
		if (!coordsOk(x,y)) return false;
		if (grid[x][y].first==-1) return true;
		TilePos directions[6];
		rotateTile(grid[x][y].first,grid[x][y].second,directions);
		TilePos opp = oppositeDir(dir);
		int i = 0;
		for (; i < 6; i++) {
			if (directions[i]==THIS) continue;
			if (directions[i]==opp) {
				dir = directions[i%2?i-1:i+1];
				break;
			}
		}
		if (i==6) return false;
	}
}

void BoardGame::tryPiece(int n, int tile, int rotation, TilePos problems[], int &pos) {
	PlayerStatus &ps = player[n];
	TilePos directions[6];
	rotateTile(tile,rotation,directions);
	bool startFound = false;
	TilePos startDir = oppositeDir(ps.direction);
	pos = 0;

	int gridx = ps.posX;
	int gridy = ps.posY;
	nextPos(gridx,gridy,ps.direction); // gridx and gridy are the coords for the tile to check for validity
	if (!coordsOk(gridx,gridy)) {
		problems[pos++] = startDir;
	}
	for (int i = 0; i < 6; i++) {
		if (directions[i]==THIS) break;
		if (directions[i]==startDir) startFound = true;
		int thisx = gridx, thisy = gridy;
		nextPos(thisx,thisy,directions[i]);
		if (!coordsOk(thisx,thisy)) {
			problems[pos++] = directions[i];
//			printf("problem type 1, direction %s\n",tileNames[directions[i]]);
			continue;
		}
		if (grid[thisx][thisy].first!=-1) {
			// check that the connections match
			TilePos d2[6];
			rotateTile(grid[thisx][thisy].first,grid[thisx][thisy].second,d2);
			TilePos opp = oppositeDir(directions[i]);
			int j = 0;
			for (; j < 6; j++) {
				if (opp==d2[j]) break;
			}
			if (j==6) {
				problems[pos++] = directions[i];
//				printf("problem type 2, direction %s\n",tileNames[directions[i]]);
			}
		}
	}

	int origTile = grid[gridx][gridy].first; // might be ENDTILE
	grid[gridx][gridy].first = tile;
	grid[gridx][gridy].second = rotation;
	for (int i = 0; i < 4; i++) {
		if (!player[i].active) continue;
		if (player[i].finished) continue;
		int thisx = player[i].posX;
		int thisy = player[i].posY;
		nextPos(thisx,thisy,player[i].direction);
		if (thisx==gridx && thisy==gridy) { // this tile is directly in front of a player
			// check that this player can advance from here
			if (!testAdvance(player[i].posX,player[i].posY,player[i].direction)) {
				problems[pos++] = oppositeDir(player[i].direction);
//				printf("problem type 3, direction %s\n",tileNames[player[i].direction]);
			}
		}
	}
	grid[gridx][gridy].first = origTile;

	if (!startFound) {
		problems[pos++] = startDir;
//		printf("problem type 4, direction %s\n",tileNames[startDir]);
	}
}

void BoardGame::validate() {
	TilePos problems[15];
	int pos;
	tryPiece(curPlayer,selectedTile,tileRotation,problems,pos);
	if (pos==0) {
		tileIsOk = true;
		if (problemHighlight!=NULL) SDL_FreeSurface(problemHighlight);
		problemHighlight = NULL;
	} else {
		tileIsOk = false;
		problems[pos++] = THIS;
		if (problemHighlight!=NULL) SDL_FreeSurface(problemHighlight);
		problemHighlight = Effects::createHighlight(problems);
	}
}

bool BoardGame::tryPiece(int n, int tile, int rot) {
	if (numTiles[tile]==0) return false;
	TilePos problems[15];
	int pos;
	tryPiece(n,tile,rot,problems,pos);
	return pos==0;
}

bool BoardGame::isPressable(int n) {
	if (selectedTile==-1) return false;
	if (n==1 && !tileIsOk) return false;
	return true;
}

void BoardGame::newTurn() {

	if (player[curPlayer].finished) {
		nextPos(player[curPlayer].posX,player[curPlayer].posY,player[curPlayer].direction);
		player[curPlayer].lengths.clear();
	}

	state = WAITING;
	int playing = 0;
	for (int i = 0; i < 4; i++) {
		PlayerStatus &ps = player[i];
		if (!ps.active) continue;
		if (!ps.finished) playing++;
		else continue;
		int n = ps.lengths.size()/2;
		ps.passedBlocks -= ps.blocksAdded;
		for (int j = 0; j < n; j++) {
			advance(ps.posX,ps.posY,ps.direction);
			ps.passedBlocks++;
		}
		redrawLabels2 = true;
		ps.blocksAdded = 0;
		ps.lengths.clear();
		ps.totLength = 0;
	}
	animStartTime = lastTime;

	if (playing==0) {
		state = FINISHED;
	}

	while (true) {
		if (playing==0) break;
		curPlayer++;
		if (curPlayer==4) curPlayer = 0;
		PlayerStatus &ps = player[curPlayer];
		if (!ps.active) continue;
		if (ps.finished) continue;
		bool ok = false;
		for (int i = 0; i < 10; i++) {
			for (int j = 0; j < 6; j++) {
				if (tryPiece(curPlayer,i,j)) {
					ok = true;
					break;
				}
			}
			if (ok) break;
		}
		if (!ok) {
			playing--;
			ps.finished = true;

			int gridx = ps.posX;
			int gridy = ps.posY;
			nextPos(gridx,gridy,ps.direction);
			int rot = getRotation(ps.direction);
			grid[gridx][gridy].first = ENDTILE;
			grid[gridx][gridy].second = rot;
			if (curTile!=NULL) SDL_FreeSurface(curTile);
			curTile = Effects::rotate(endTile,2*M_PI*rot/6,true);
//			SDL_Surface *rotTile = Effects::rotate(endTile,2*M_PI*rot/6,true);
//			SDL_Rect rect;
//			getDrawRect(gridx,gridy,rotTile,rect);
//			addTileOffset(rect,10,rot);
//			SDL_BlitSurface(rotTile,NULL,bkg,&rect);
//			SDL_FreeSurface(rotTile);
//			se.redrawRect(rect);


//			state = ANIMATING;
			state = FADETILE;
			animMaxLen = 0;
			ps.blocksAdded = 0;
			ps.lengths.clear();
			ps.totLength = 0;
			int nr;
			bool opposite;
			int tile = grid[ps.posX][ps.posY].first;
			int rotation = grid[ps.posX][ps.posY].second;
			findPath(tile,rotation,ps.direction,nr,opposite);
			CompSpline *spline = getSpline(tile,rotation,nr,opposite);
			float len = spline->getLength();
			delete spline;
			ps.totLength += len;
			ps.lengths.push_back(len);

			findPath(0,rot,ps.direction,nr,opposite);
			spline = getSpline(0,rot,nr,opposite);
			len = spline->getLength();
			delete spline;
			ps.totLength += len;
			ps.lengths.push_back(len);
			animMaxLen = ps.totLength;

			animFade = false;
			animStartTime = lastTime;

		}
		break;
	}


	if (state==FINISHED) {
		int maxPassed = 0;
		bool maxSet = false;
		for (int i = 0; i < 4; i++) {
			PlayerStatus &ps = player[i];
			if (!ps.active) continue;
			if (!maxSet || ps.passedBlocks>maxPassed) {
				maxPassed = ps.passedBlocks;
				maxSet = true;
			}
		}
		result = 0;
		for (int i = 0; i < 4; i++) {
			PlayerStatus &ps = player[i];
			if (!ps.active) continue;
			result |= Uint8(ps.passedBlocks)<<(8*i);
			if (ps.passedBlocks==maxPassed)
				ps.winner = true;
		}
		animStartTime = lastTime;
	}

}

void BoardGame::grayTile(int n) {
	SDL_Rect grayRect;
	grayRect.x = tilePos[n].x;
	grayRect.y = tilePos[n].y;
	grayRect.w = tileBkg->w;
	grayRect.h = tileBkg->h;
	SDL_BlitSurface(board,&grayRect,bkg,&grayRect);

	SDL_Surface *grayBkg = Effects::gray(tileBkg);
	SDL_BlitSurface(grayBkg,NULL,bkg,&grayRect);
	SDL_Surface *img = Effects::gray(tile[n]);
	SDL_Rect r = grayRect;
	addTileOffset(r,n,0,false);
	SDL_BlitSurface(img,NULL,bkg,&r);
	SDL_FreeSurface(img);

	for (int i = 0; i < 10; i++) {
		if (i==n) continue;

		SDL_Rect thisTile;
		thisTile.x = tilePos[i].x;
		thisTile.y = tilePos[i].y;
		thisTile.w = tileBkg->w;
		thisTile.h = tileBkg->h;
		SDL_Rect r1, r2;

		if (!intersect(thisTile,grayRect,r1,r2)) continue;
		SDL_Surface *ptr = tileBkg;
		if (numTiles[i]==0) ptr = grayBkg;
		SDL_BlitSurface(ptr,&r2,bkg,&r1);

		addTileOffset(thisTile,i,0,false);
		if (!intersect(thisTile,grayRect,r1,r2)) continue;
		img = tile[i];
		if (numTiles[i]==0) img = Effects::gray(img);
		SDL_BlitSurface(img,&r2,bkg,&r1);
		if (numTiles[i]==0) SDL_FreeSurface(img);
	}
	se.redrawRect(grayRect);
	SDL_FreeSurface(grayBkg);
}

void BoardGame::buttonPress(int n) {

	if (!isPressable(n)) return;
	switch (n) {
	case 1: { // OK
		int gridx = player[curPlayer].posX;
		int gridy = player[curPlayer].posY;
		// put curTile into bkg
		nextPos(gridx,gridy,player[curPlayer].direction);
		grid[gridx][gridy].first = selectedTile;
		grid[gridx][gridy].second = tileRotation;
		SDL_Rect rect;
		getDrawRect(gridx,gridy,curTile,rect);
		addTileOffset(rect,selectedTile,tileRotation);
		SDL_BlitSurface(curTile,NULL,bkg,&rect);
		se.redrawRect(rect);

		animMaxLen = 0;
		for (int i = 0; i < 4; i++) {
			PlayerStatus &ps = player[i];
			if (!ps.active) continue;
			if (ps.finished) continue;
			ps.blocksAdded = 0;
			int x = ps.posX;
			int y = ps.posY;
			TilePos dir = ps.direction;
			ps.lengths.clear();
			ps.totLength = 0;
			int lastx = x;
			int lasty = y;
			TilePos lastdir = dir;
			while (advance(x,y,dir)) {
				int tile = grid[lastx][lasty].first;
				int rotation = grid[lastx][lasty].second;
				int nr;
				bool opposite;
				findPath(tile,rotation,lastdir,nr,opposite);
				CompSpline *spline = getSpline(tile,rotation,nr,opposite);
				float len = spline->getLength();
				delete spline;
				ps.totLength += len;
				ps.lengths.push_back(len);

				tile = grid[x][y].first;
				rotation = grid[x][y].second;
				findPath(tile,rotation,dir,nr,opposite);
				spline = getSpline(tile,rotation,nr,opposite);
				len = spline->getLength();
				delete spline;
				ps.totLength += len;
				ps.lengths.push_back(len);

				lastx = x;
				lasty = y;
				lastdir = dir;
			}
			if (ps.totLength>animMaxLen) animMaxLen = ps.totLength;
		}

		SDL_FreeSurface(curTile);
		curTile = NULL;

		numTiles[selectedTile]--;
		if (numTiles[selectedTile]==0) grayTile(selectedTile);
		redrawLabels = true;
		selectedTile = -1;
		tileRotation = 0;
		tileIsOk = false;
		state = ANIMATING;
		animFade = true;
		animModeStart = lastTime;

		} break;
	case 0: // rotate left
		tileRotation--;
		if (tileRotation<0) tileRotation += 6;
		break;
	case 2: // rotate right
		tileRotation++;
		if (tileRotation>=6) tileRotation -= 6;
		break;
	}
	if (n==0 || n==2) {
		validate();
	}
}

void BoardGame::updateButton(int n, int state) {
	if (!isPressable(n)) state = 0;
	SDL_Rect &r = buttons[n].r;
	SDL_BlitSurface(board,&r,bkg,&r);
	SDL_BlitSurface(buttons[n].surf[state],NULL,bkg,&r);
	for (int i = 0; i < 3; i++) {
		if (i==n) continue;
		SDL_Rect r1, r2;
		if (!intersect(buttons[i].r,buttons[n].r,r1,r2)) continue;
		SDL_BlitSurface(buttons[i].surf[0],&r2,bkg,&r1);
	}
	se.redrawRect(r);
}

void BoardGame::setPointer(int n) {
	if (wrapper) {
		wrapper->setPointer(n);
	} else {
		if (n>=0 && n<3) {
			curPointer = pointer[n];
			curOffset = pointerOff[n];
		}
	}
}

void BoardGame::handleEvent(SDL_Event &event) {
	if (event.type==SDL_KEYDOWN) {
		if (isPaused) return;
		SDLKey sym = event.key.keysym.sym;
		if (sym==SDLK_ESCAPE) {
			if (wrapper && state!=FINISHED)
				wrapper->showQuitDialog();
			else
				quit = true;
		}
	} else if (event.type==SDL_MOUSEBUTTONDOWN) {
		if (isPaused) return;
		if (state==WAITING) {
			int x = event.button.x;
			int y = event.button.y;
			int button = findButton(x,y);
			if (button!=-1) {
				buttondown = button;
				if (isPressable(button)) setPointer(2);
				return;
			}
			int n = findTile(x,y);
			if (n==-1) return;
			if (numTiles[n]==0) return;
			buttondownTile = n;
			setPointer(2);
			if (n==selectedTile) return;
			selectedTile = n;
			validate();

		} else if (state==ANIMATING) {
			newTurn();
		} else if (state==FINISHED) {
			quit = true;
		} else if (state==FADETILE) {
			mergeEndTile();
			state = ANIMATING;
			animStartTime = lastTime;
		}
	} else if (event.type==SDL_MOUSEBUTTONUP) {
		if (isPaused) return;
		if (state==WAITING) {
			int button = findButton(event.button.x,event.button.y);
			if (buttondown>=0 && button==buttondown) {
				buttonPress(button);
			}
			if (buttondown>=0 || buttondownTile>=0) {
				buttondown = -1;
				buttondownTile = -1;
				// send fake mousemotion to update pointer type
				SDL_Event e2;
				e2.type = SDL_MOUSEMOTION;
				e2.motion.x = event.button.x;
				e2.motion.y = event.button.y;
				handleEvent(e2);
			}
		} else if (state==ANIMATING) {
		}
	} else if (event.type==SDL_MOUSEMOTION) {
		curMousePos.x = event.motion.x;
		curMousePos.y = event.motion.y;
		if (isPaused) return;
		if (state==WAITING) {
			bool clickable = false;
			int button = findButton(event.motion.x,event.motion.y);
			int tile = findTile(event.motion.x,event.motion.y);
			if (tile!=-1 && numTiles[tile]>0) clickable = true;
			if (button!=-1 && isPressable(button)) clickable = true;
			buttonover = button;

			if (buttondown>=0) {
				setPointer(buttonover==buttondown?2:0);
			} else if (buttondownTile>=0) {
				setPointer(tile==buttondownTile?2:0);
			} else {
				setPointer(clickable?1:0);
			}
		} else if (state==ANIMATING || state==FADETILE) {
			setPointer(0);
			buttonover = -1;
			buttondown = -1;
		} else if (state==FINISHED) {
			setPointer(0);
		}
	}
}

void BoardGame::mergeRects(SDL_Rect &main, const SDL_Rect &r2) {
	if (r2.x<main.x) {
		main.w += main.x-r2.x;
		main.x = r2.x;
	}
	if (r2.y<main.y) {
		main.h += main.y-r2.y;
		main.y = r2.y;
	}
	if (r2.x+r2.w>main.x+main.w)
		main.w = r2.x+r2.w-main.x;
	if (r2.y+r2.h>main.y+main.h)
		main.h = r2.y+r2.h-main.y;
}

bool BoardGame::rectsIntersect(const SDL_Rect &r1, const SDL_Rect &r2) {
	if (r1.w == 0 || r1.h == 0 || r2.w == 0 || r2.h == 0) return false;
	if (r1.x<r2.x) {
		if (r1.x+r1.w>r2.x) {
			if (r1.y<r2.y) {
				if (r1.y+r1.h>r2.y) return true;
			} else {
				if (r2.y+r2.h>r1.y) return true;
			}
		}
	} else {
		if (r2.x+r2.w>r1.x) {
			if (r1.y<r2.y) {
				if (r1.y+r1.h>r2.y) return true;
			} else {
				if (r2.y+r2.h>r1.y) return true;
			}
		}
	}
	return false;
}

bool BoardGame::intersect(const SDL_Rect &r1, const SDL_Rect &r2, SDL_Rect &global, SDL_Rect &local) {
	global.x = max(r1.x,r2.x);
	global.y = max(r1.y,r2.y);
	int w = min(r1.x+r1.w,r2.x+r2.w)-global.x;
	int h = min(r1.y+r1.h,r2.y+r2.h)-global.y;
	global.w = w;
	global.h = h;
	local.x = global.x - r1.x;
	local.y = global.y - r1.y;
	local.w = global.w;
	local.h = global.h;
	if (w<=0 || h<=0) return false;
	return true;
}

void BoardGame::findPath(int tile, int rotation, TilePos dir, int &nr, bool &opposite) {
	TilePos directions[6];
	rotateTile(tile,rotation,directions);
	for (int i = 0; i < 6; i++) {
		if (directions[i]==dir) {
			nr = i/2;
			opposite = (i%2)==0;
			return;
		}
	}
	nr = -1;
}

SDL_Surface *BoardGame::createSplineTest(int x, int y, int w, int h) {
	SDL_Surface *out = SDL_CreateRGBSurface(SDL_SWSURFACE|SDL_SRCALPHA,w,h,32,0x00FF0000,0x0000FF00,0x000000FF,0xFF000000);
	SDL_FillRect(out,NULL,0);
	if (SDL_MUSTLOCK(out)) SDL_LockSurface(out);
	Uint32 *outptr = (Uint32*)out->pixels;

	for (int i = 0; i < 10; i++) {
		for (int j = 0; j < 3; j++) {
			CompSpline *spline = getSpline(i,0,j,false);
			if (spline==NULL) break;
			for (float f = 0; f <= 1; f += 0.01) {
				FPoint pos;
				spline->calc(f,&pos);
				pos.x += tilePos[i].x-x;
				pos.y += tilePos[i].y-y;
				int xpos = int(pos.x+0.5);
				int ypos = int(pos.y+0.5);
				if (xpos<0 || xpos>=w) continue;
				if (ypos<0 || ypos>=h) continue;
				outptr[ypos*out->pitch/4+xpos] = 0xFFFF0000;
			}
			delete spline;
		}
	}

	if (SDL_MUSTLOCK(out)) SDL_UnlockSurface(out);
	return out;
}

void BoardGame::updateTractor(float pos, int n, SDL_Rect &out) {
	PlayerStatus &ps = player[n];
	int tile, rotation;
	TilePos dir;
	int gridx = ps.posX, gridy = ps.posY;
	tile = grid[gridx][gridy].first;
	rotation = grid[gridx][gridy].second;
	dir = ps.direction;

	pos *= ps.totLength;
	int add = 0;
	for (unsigned int i = 0; i < ps.lengths.size(); i++) {
		if (i%2==1) {
			if (ps.finished) {
				nextPos(gridx,gridy,dir);
			} else {
				advance(gridx,gridy,dir);
			}
			add++;
		}
		if (pos<ps.lengths[i] || i==ps.lengths.size()-1) {
			pos = 0.5*pos/ps.lengths[i];
			if (pos>0.5) pos = 0.5;
			if (i%2==0) pos += 0.5;
			break;
		} else {
			pos -= ps.lengths[i];
		}
	}
	if (add>0 && add>ps.blocksAdded && !ps.finished) {
		ps.passedBlocks += add-ps.blocksAdded;
		ps.blocksAdded = add;
		redrawLabels2 = true;
	}
	tile = grid[gridx][gridy].first;
	rotation = grid[gridx][gridy].second;
	if (ps.lengths.size()==0) {
		pos = 0.5;
	}
	int nr;
	bool opposite;
	if (tile==ENDTILE) {
		tile = 0;
		rotation = getRotation(ps.direction);
	}
	findPath(tile,rotation,dir,nr,opposite);
	SplineInfo si(tile,rotation,nr,opposite);
	if (ps.curSpline==NULL || si!=ps.splineInfo) {
		if (ps.curSpline!=NULL) delete ps.curSpline;
		ps.curSpline = getSpline(tile,rotation,nr,opposite);
		ps.splineInfo = si;
	}
	FPoint center, center2;
#define MOVEMENTDIFF 0.0001
	if (pos+MOVEMENTDIFF>1) {
		ps.curSpline->calc(pos-MOVEMENTDIFF,&center);
		ps.curSpline->calc(pos,&center2);
	} else {
		ps.curSpline->calc(pos,&center);
		ps.curSpline->calc(pos+MOVEMENTDIFF,&center2);
	}
	FPoint diff = center2-center;
	float angle = M_PI/2 + atan2(diff.y,diff.x);
	if (ps.rotImg==NULL || ps.rot!=angle) {
		if (ps.rotImg!=NULL) SDL_FreeSurface(ps.rotImg);
		ps.rotImg = Effects::rotate(tractor[n],angle,true);
		ps.rot = angle;
	}
	getCoords(gridx,gridy,out.x,out.y);
	out.x += int(center.x+0.5+cos(angle)*tOffset[n].x-sin(angle)*tOffset[n].y)-ps.rotImg->w/2;
	out.y += int(center.y+0.5+cos(angle)*tOffset[n].y+sin(angle)*tOffset[n].x)-ps.rotImg->h/2;
	out.w = ps.rotImg->w;
	out.h = ps.rotImg->h;
}

void BoardGame::addTileOffset(SDL_Rect &rect, int n, int rot, bool middle) {
	float angle = 2*M_PI*rot/6;
	if (!middle) {
		rect.x += (tileBkg->w-tile[n]->w)/2;
		rect.y += (tileBkg->h-tile[n]->h)/2;
	}
	rect.x += int(cos(angle)*tileOffset[n].x-sin(angle)*tileOffset[n].y);
	rect.y += int(cos(angle)*tileOffset[n].y+sin(angle)*tileOffset[n].x);
}

void BoardGame::mergeEndTile() {
	int gridx = player[curPlayer].posX;
	int gridy = player[curPlayer].posY;
	nextPos(gridx,gridy,player[curPlayer].direction);
	int rot = getRotation(player[curPlayer].direction);
	SDL_Rect rect;
	getDrawRect(gridx,gridy,curTile,rect);
	addTileOffset(rect,10,rot);

	SDL_BlitSurface(curTile,NULL,bkg,&rect);
	SDL_FreeSurface(curTile);
	curTile = NULL;
	se.redrawRect(rect);
}

void BoardGame::update(Uint32 time, SDL_Surface *screen) {
	if (isPaused) {
		animStartTime += time-lastTime;
		animModeStart += time-lastTime;
	}
	lastTime = time;
	if (state==FINISHED && wrapper!=NULL) {
		if (time-animStartTime>3500) {
			quit = true;
		}
	}


	if (last.buttonover!=buttonover || last.buttondown!=buttondown) {
		if (last.buttonover>=0) {
			updateButton(last.buttonover,0);
		}
		if (buttonover>=0) {
			if (buttondown==buttonover) {
				updateButton(buttonover,2);
			} else {
				if (buttondown>=0) {
					updateButton(buttonover,0);
				} else {
					updateButton(buttonover,1);
				}
			}
		}
	}




	int textId[10];
	int tractorId[4];
	int tractorTextId[4];
	int tractorHighlightId[4];
	for (int i = 0; i < 4; i++) {
		tractorId[i] = se.getNextId();
		tractorHighlightId[i] = se.getNextId();
	}
	int selectedHighlightId = se.getNextId();
	int highlightId = se.getNextId();
	int problemId = se.getNextId();
	int pointerId = se.getNextId();
	int curTileId = se.getNextId();

	SDL_Surface *selectedHighlight = NULL;
	if (selectedTile>=0) {
		SDL_Rect r;
		r.x = tilePos[selectedTile].x+(tileBkg->w-highlightSize.w)/2;
		r.y = tilePos[selectedTile].y+(tileBkg->h-highlightSize.h)/2;
		r.w = highlightSize.w;
		r.h = highlightSize.h;
		selectedHighlight = Effects::createHighlight(THIS,192,0x00FF0000);
		se.addSprite(selectedHighlightId,new Sprite(selectedHighlight,r,false));
	}

	for (int i = 0; i < 10; i++) {
		textId[i] = se.getNextId();
		SDL_Rect textSize, drawPos;
		char textBuffer[10];
		sprintf(textBuffer,"%d",numTiles[i]);
		int w, h;
		TTF_SizeText(font,textBuffer,&w,&h);
		textSize.w = w;
		textSize.h = h;
		drawPos.x = tilePos[i].x+tileBkg->w/2-textSize.w/2;
		drawPos.y = tilePos[i].y-textSize.h-0;
		drawPos.w = textSize.w;
		drawPos.h = textSize.h;
		se.addSprite(textId[i],new Sprite(NULL,drawPos,redrawLabels));
	}
	redrawLabels = false;
	for (int i = 0; i < 4; i++) {
		tractorTextId[i] = se.getNextId();
		PlayerStatus &ps = player[i];
		if (!ps.active) continue;
		char textBuffer[10];
		sprintf(textBuffer,"%d",ps.passedBlocks);
		int w, h;
		TTF_SizeText(font,textBuffer,&w,&h);
		SDL_Rect textSize, drawPos;
		textSize.w = w;
		textSize.h = h;
		drawPos.x = textPos[i].x-textSize.w/2;
		drawPos.y = textPos[i].y;
		drawPos.w = textSize.w;
		drawPos.h = textSize.h;
		se.addSprite(tractorTextId[i],new Sprite(NULL,drawPos,redrawLabels2));
	}
	redrawLabels2 = false;

	bool redrawCurTile = false;
	if (state==WAITING && selectedTile>=0 && (selectedTile!=last.selectedTile || tileRotation!=last.tileRotation)) {
		if (curTile!=NULL) SDL_FreeSurface(curTile);
		curTile = Effects::rotate(tile[selectedTile],2*M_PI*tileRotation/6,true);
		redrawCurTile = true;
	}

#define TILEFADELENGTH 200
	if (state==FADETILE) {
		if (time-animStartTime>TILEFADELENGTH) {
			mergeEndTile();
			state = ANIMATING;
			animStartTime = time;
		}
	}
#define HIGHLIGHTFADELENGTH 500
	if (state==ANIMATING && animFade) {
		int len = 2*HIGHLIGHTFADELENGTH;
		Uint32 max = ((animModeStart-animStartTime)/len)*len + len;
		if (time-animStartTime>max) {
			animStartTime = time;
			animFade = false;
		}
	}
#define MOVELENGTH 10
	if (state==ANIMATING && !animFade) {
		if (time-animStartTime>MOVELENGTH*animMaxLen) {
			newTurn();
		}
	}
	SDL_Surface *highlight = NULL;
	SDL_Surface *fadeTile = NULL;
	if (state==WAITING || animFade) {
		Uint32 blinkLength = HIGHLIGHTFADELENGTH;
		int pos = 192*((time-animStartTime)%(2*blinkLength))/blinkLength;
		if ((time-animStartTime)%(2*blinkLength)>blinkLength)
			pos = 192*(2*blinkLength-(time-animStartTime)%(2*blinkLength))/blinkLength;


		PlayerStatus &ps = player[curPlayer];
		SDL_Rect hrect;
		int gridx = ps.posX, gridy = ps.posY;

		nextPos(gridx,gridy,ps.direction);
		highlight = Effects::createHighlight(THIS, pos, ps.highlightColor);
		getDrawRect(gridx,gridy,highlight,hrect);


		if (curTile!=NULL) {
			SDL_Rect r;
			getDrawRect(gridx,gridy,curTile,r);
			addTileOffset(r,selectedTile,tileRotation);
			se.addSprite(curTileId,new Sprite(curTile,r,redrawCurTile));
		}

		for (int i = 0; i < 4; i++) {
			if (!player[i].active) continue;
			SDL_Rect r;
			updateTractor(0,i,r);
			se.addSprite(tractorId[i],new Sprite(player[i].rotImg,r,false));
		}

		se.addSprite(highlightId,new Sprite(highlight,hrect,true));

		if (problemHighlight!=NULL) {
			SDL_Rect r;
			getDrawRect(gridx,gridy,problemHighlight,r);
			se.addSprite(problemId,new Sprite(problemHighlight,r,false));
		}
	} else if (state==FINISHED) {
		Uint32 blinkLength = HIGHLIGHTFADELENGTH/2;
		int pos = 192*((time-animStartTime)%(2*blinkLength))/blinkLength;
		if ((time-animStartTime)%(2*blinkLength)>blinkLength)
			pos = 192*(2*blinkLength-(time-animStartTime)%(2*blinkLength))/blinkLength;

		for (int i = 0; i < 4; i++) {
			PlayerStatus &ps = player[i];
			if (!ps.active) continue;
			SDL_Rect r;
			updateTractor(0,i,r);
			se.addSprite(tractorId[i],new Sprite(ps.rotImg,r,false));

			if (ps.winner) {
				SDL_Surface *surf = Effects::createHighlight(THIS, pos, ps.highlightColor);
				getDrawRect(ps.posX,ps.posY,surf,r);
				se.addSprite(tractorHighlightId[i],new Sprite(surf,r,true));
			}
		}
	} else if (state==FADETILE) {
		int gridx = player[curPlayer].posX;
		int gridy = player[curPlayer].posY;
		nextPos(gridx,gridy,player[curPlayer].direction);
		int rot = getRotation(player[curPlayer].direction);

		SDL_Rect r;
		getDrawRect(gridx,gridy,curTile,r);
		addTileOffset(r,10,rot);
		int alpha = 256*(time-animStartTime)/TILEFADELENGTH;
		if (alpha<0) alpha = 0;
		if (alpha>256) alpha = 256;
		fadeTile = Effects::multAlpha(curTile,alpha);
		se.addSprite(curTileId,new Sprite(fadeTile,r,true));

		for (int i = 0; i < 4; i++) {
			PlayerStatus &ps = player[i];
			if (!ps.active) continue;
			SDL_Rect r;
			updateTractor(0,i,r);
			se.addSprite(tractorId[i],new Sprite(ps.rotImg,r,false));
		}
	} else {
		for (int i = 0; i < 4; i++) {
			PlayerStatus &ps = player[i];
			if (!ps.active) continue;
			float pos = 0;
#ifdef MOVESYNCEND
			if (ps.totLength>0 && time-animStartTime>(animMaxLen-ps.totLength)*MOVELENGTH)
				pos = float(time-animStartTime-(animMaxLen-ps.totLength)*MOVELENGTH)/(MOVELENGTH*ps.totLength);
#else
			if (ps.totLength>0)
				pos = float(time-animStartTime)/(MOVELENGTH*ps.totLength);
			if (pos>1) pos = 1;
#endif
			SDL_Rect r;
			updateTractor(pos,i,r);
			se.addSprite(tractorId[i],new Sprite(ps.rotImg,r,true));
		}
	}


	SDL_Rect rect;


#ifdef SPLINETEST
	int splineTestId = se.getNextId();
	rect.x = tilePos[0].x;
	rect.y = tilePos[0].y;
	rect.w = tilePos[9].x+tile[9]->w-rect.x;
	rect.h = tilePos[9].y+tile[9]->h-rect.y;
	se.addSprite(splineTestId,new Sprite(NULL,rect,false));
#endif

	if (wrapper==NULL) {
		rect.x = curMousePos.x - curOffset.x;
		rect.y = curMousePos.y - curOffset.y;
		rect.w = curPointer->w;
		rect.h = curPointer->h;
		bool redraw = false;
		if (curPointer!=last.curPointer) redraw = true;
		se.addSprite(pointerId,new Sprite(curPointer,rect,redraw));
	}



	se.calcUpdates();

#ifdef SPLINETEST
	SDL_Surface *splineTest = NULL;
	if (se.getSprite(splineTestId)->update) {
		Sprite *s = se.getSprite(splineTestId);
		splineTest = createSplineTest(s->pos.x+s->updateRect.x,s->pos.y+s->updateRect.y,s->updateRect.w,s->updateRect.h);
		s->img = splineTest;
		s->relative = true;
	}
#endif

	for (int i = 0; i < 10; i++) {
		Sprite *s = se.getSprite(textId[i]);
		if (s->update) {
			char textBuffer[10];
			sprintf(textBuffer,"%d",numTiles[i]);
			SDL_Color tc = { 0x00, 0x00, 0x00, 0 };
			if (numTiles[i]==0) {
				tc.r = 0x99;
				tc.g = 0x99;
				tc.b = 0x99;
			}
			s->img = TTF_RenderText_Blended(font,textBuffer,tc);
		}
	}
	for (int i = 0; i < 4; i++) {
		Sprite *s = se.getSprite(tractorTextId[i]);
		if (s && s->update) {
			PlayerStatus &ps = player[i];
			char textBuffer[10];
			sprintf(textBuffer,"%d",ps.passedBlocks);
			SDL_Color tc;
			tc.r = (ps.highlightColor>>16)&0xFF;
			tc.g = (ps.highlightColor>>8)&0xFF;
			tc.b = ps.highlightColor&0xFF;
			s->img = TTF_RenderText_Blended(font,textBuffer,tc);
		}
	}

	se.update(screen,bkg);

	for (int i = 0; i < 10; i++) {
		SDL_FreeSurface(se.getSprite(textId[i])->img);
	}
	for (int i = 0; i < 4; i++) {
		Sprite *s = se.getSprite(tractorTextId[i]);
		if (s && s->img)
			SDL_FreeSurface(s->img);
		s = se.getSprite(tractorHighlightId[i]);
		if (s && s->img)
			SDL_FreeSurface(s->img);
	}
	SDL_FreeSurface(highlight);
	SDL_FreeSurface(selectedHighlight);
	SDL_FreeSurface(fadeTile);

#ifdef SPLINETEST
	SDL_FreeSurface(splineTest);
#endif


	se.reset();

	last = *this;

}

bool BoardGame::finished() {
	return quit;
}

void BoardGame::redraw() {
	se.redrawAll();
}

void BoardGame::setUpdatedRect(void (*func)(const SDL_Rect &r)) {
	Game::setUpdatedRect(func);
	se.setUpdatedRect(func);
}

void BoardGame::pause(bool p) {
	isPaused = p;
//	buttonover = -1;
//	buttondown = -1;
	if (isPaused) {
		setPointer(0);
	} else {
		SDL_Event event;
		event.type = SDL_MOUSEMOTION;
		event.motion.x = curMousePos.x;
		event.motion.y = curMousePos.y;
		handleEvent(event);
	}
}

void BoardGame::getRects(vector<SDL_Rect> &list) {
	se.getRects(list);
}

int BoardGame::getResult() {
	return result;
}


